/* Testing Code */

#include <limits.h>
#include <math.h>

/* Routines used by floation point test code */

/* Convert from bit level representation to floating point number */
float u2f(unsigned u) {
  union {
    unsigned u;
    float f;
  } a;
  a.u = u;
  return a.f;
}

/* Convert from floating point number to bit-level representation */
unsigned f2u(float f) {
  union {
    unsigned u;
    float f;
  } a;
  a.f = f;
  return a.u;
}

/* Copyright (C) 1991-2016 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.  */
/* This header is separate from features.h so that the compiler can
   include it implicitly at the start of every compilation.  It must
   not itself include <features.h> or any other header that includes
   <features.h> because the implicit include comes before any feature
   test macros that may be defined in a source file before it first
   explicitly includes a system header.  GCC knows the name of this
   header in order to preinclude it.  */
/* glibc's intent is to support the IEC 559 math functionality, real
   and complex.  If the GCC (4.9 and later) predefined macros
   specifying compiler intent are available, use them to determine
   whether the overall intent is to support these features; otherwise,
   presume an older compiler has intent to support these features and
   define these macros by default.  */
/* wchar_t uses Unicode 8.0.0.  Version 8.0 of the Unicode Standard is
   synchronized with ISO/IEC 10646:2014, plus Amendment 1 (published
   2015-05-15).  */
/* We do not support C11 <threads.h>.  */
int test_bitOr(int x, int y)
{
  return x|y;
}
int test_isNegative(int x) {
  return x < 0;
}
int test_isLessTen(int x) {
  return x < 10;
}
int test_ezFiveFourths(int x) {
    return (x*5)/4;
}
int test_everyThirdBitOne(void) {
    return 0x24924924;
}
int test_byteEndSwap(int x) {
    /* little endiamachine */
    /* least significant byte stored first */
    unsigned ux = x;
    return (((ux >> 8) << 16) >> 8) | ((ux & 0xFF000000) >> 24) |
  ((ux & 0x000000FF) << 24);
}
int test_numOddBytes(int x) {
    int total = 0;
    for (int i = 0; i < sizeof(int); i++) {
        total += x & 0x1;
        x = x >> 8;
    }
    return total;
}
int test_subOK(int x, int y)
{
  long long ldiff = (long long) x - y;
  return ldiff == (int) ldiff;
}
int test_squaredOK(int x) {
    int squared_int = x * x;
    long long squared64 = (long long)x * x;
    return squared_int == squared64;
}
/* adapted from CMU lab puzzle bitParity.c */
int test_bitParity(int x) {
    int result = 0;
    int i;
    for (i = 0; i < 32; i++)
        result += (x >> i) & 0x1;
    return result % 2;
}
int test_byteParity(int x) {
    int byte = 0x0;
    int res = 0x0;
    int i;
    for (i = 0; i < sizeof(int); i++) {
 /* isolate the 7 least sig bits of each byte */
        byte = (x >> (i * 8)) & 0x7F;
 /* calculate bit parity and OR it into most sig bit of byte */
 res |= (byte | (test_bitParity(byte) << 7)) << (i * 8);
    }
    return res;
}
unsigned test_floatTimesTwo(unsigned uf) {
    float f = u2f(uf);
    if (isnan(f))
 return uf;
    f = f * 2;
    return f2u(f);
}
unsigned test_roundFloat(unsigned uf) {
    float fl = 0.0;
    float result = 0;
    float frac = 0.0;
    /* convert integer float representation to real float */
    fl = u2f(uf);
    if (isnan(fl) || isinf(fl))
 return uf;
    /* Leave negative zero unchanged */
    if (uf == 0x80000000)
 return uf;
    if (fl + 0.5 == fl) {
 /* So big that only integers can be represented. Return unchanged. */
 return uf;
    }
    float rounded = (float)(long long)fl;
    frac = fl - rounded;
    if (frac >= 0.5)
        result = rounded + 1;
    else if (frac <= -0.5)
        result = rounded - 1;
    else
        result = rounded;
    /* convert float to integer float representation */
    return f2u(result);
}
