/*
 * CS:APP Data Lab
 *
 * <Please put your name and userid here>
 *
 * bits.c - Source file with your solutions to the Lab.
 *      This is the file you will hand in to your instructor.
 *
 * WARNING: Do not include the <stdio.h> header; it confuses the dlc
 * compiler. You can still use printf for debugging without including
 * <stdio.h>, although you might get a compiler warning. In general,
 * it's not good practice to ignore compiler warnings, but in this
 * case it's OK.

 * Instructions to Students:
 *
 * STEP 1: Read the following instructions carefully.

You will provide your solution to the Data Lab by
editing the collection of functions in this source file.

INTEGER CODING RULES:

  Replace the "return" statement in each function with one
  or more lines of C code that implements the function. Your code
  must conform to the following style:

  int Funct(arg1, arg2, ...) {
    // brief description of how your implementation works
    int var1 = Expr1;
    ...
    int varM = ExprM;

    varJ = ExprJ;
    ...
    varN = ExprN;
    return ExprR;
  }

  Each "Expr" is an expression using ONLY the following:
  1. Integer constants 0 through 255 (0xFF), inclusive. You are
    not allowed to use big constants such as 0xffffffff.
  2. Function arguments and local variables (no global variables).
  3. Unary integer operations ! ~
  4. Binary integer operations & ^ | + << >>

  Some of the problems restrict the set of allowed operators even further.
  Each "Expr" may consist of multiple operators. You are not restricted to
  one operator per line.

  You are expressly forbidden to:
  1. Use any control constructs such as if, do, while, for, switch, etc.
  2. Define or use any macros.
  3. Define any additional functions in this file.
  4. Call any functions.
  5. Use any other operations, such as &&, ||, -, or ?:
  6. Use any form of casting.
  7. Use any data type other than int.  This implies that you
   cannot use arrays, structs, or unions.


  You may assume that your machine:
  1. Uses 2s complement, 32-bit representations of integers.
  2. Performs right shifts arithmetically.
  3. Has unpredictable behavior when shifting an integer by more
   than the word size.

EXAMPLES OF ACCEPTABLE CODING STYLE:

  // pow2plus1 - returns 2^x + 1, where 0 <= x <= 31

  int pow2plus1(int x) {
   // exploit ability of shifts to compute powers of 2
   return (1 << x) + 1;
  }


  // pow2plus4 - returns 2^x + 4, where 0 <= x <= 31

  int pow2plus4(int x) {
   // exploit ability of shifts to compute powers of 2
   int result = (1 << x);
   result += 4;
   return result;
  }

FLOATING POINT CODING RULES

For the problems that require you to implent floating-point operations,
the coding rules are less strict.  You are allowed to use looping and
conditional control.  You are allowed to use both ints and unsigneds.
You can use arbitrary integer and unsigned constants.

You are expressly forbidden to:
  1. Define or use any macros.
  2. Define any additional functions in this file.
  3. Call any functions.
  4. Use any form of casting.
  5. Use any data type other than int or unsigned.  This means that you
   cannot use arrays, structs, or unions.
  6. Use any floating point data types, operations, or constants.


NOTES:
  1. Use the dlc (data lab checker) compiler (described in the handout) to
   check the legality of your solutions.
  2. Each function has a maximum number of operators (! ~ & ^ | + << >>)
   that you are allowed to use for your implementation of the function.
   The max operator count is checked by dlc. Note that '=' is not
   counted; you may use as many of these as you want without penalty.
  3. Use the btest test harness to check your functions for correctness.
  4. Use the BDD checker to formally verify your functions
  5. The maximum number of ops for each function is given in the
   header comment for each function. If there are any inconsistencies
   between the maximum ops in the writeup and in this file, consider
   this file the authoritative source.


 * STEP 2: Modify the following functions according the coding rules.

   IMPORTANT. TO AVOID GRADING SURPRISES:
   1. Use the dlc compiler to check that your solutions conform
    to the coding rules.
   2. Use the BDD checker to formally verify that your solutions produce
    the correct answers.
*/
int printf(const char* format, ... );
/* 
 * bitOr - x|y using only ~ and & 
 *   Example: bitOr(6, 5) = 7
 *   Legal ops: ~ &
 *   Max ops: 8
 *   Rating: 1
 */
int bitOr(int x, int y) {
  return 2;
}
/* 
 * isNegative - return 1 if x < 0, return 0 otherwise 
 *   Example: isNegative(-1) = 1.
 *   Legal ops: ! ~ & ^ | + << >>
 *   Max ops: 6
 *   Rating: 2
 */
int isNegative(int x) {
  return 2;
}
/*
 * isLessTen - return 1 if x < 10 (signed comparison), return 0 otherwise
 *   Example: isLessTen(5) = 1.
 *   Legal ops: ! ~ & ^ | + << >>
 *   Max ops: 10
 *   Rating: 2
 */
int isLessTen(int x) {
  return 2;
}
/*
 * ezFiveFourths - multiplies by 5/4 rounding toward 0. Should exactly
 *  duplicate effect of C expression (x*5)/4, including overflow behavior.
 *   Examples: ezFiveFourths(11) = 13
 *             ezFiveFourths(-9) = -11
 *             ezFiveFourths(1717986913)= -6 (overflow)
 *   Legal ops: ! ~ & ^ | + << >>
 *   Max ops: 10
 *   Rating: 1
 */
int ezFiveFourths(int x) {
    return 2;
}
/*
 * everyThirdBitOne - Return the integer constructed by setting every third bit
 * starting with bit2 to 1, e.g. bit2, bit5, bit8, ... all equal 1 and all
 * other bits 0. You may not simply return a large constant, rather start with
 * a small constant and build up the integer.
 *   Legal ops: ! ~ & ^ | + << >>
 *   Max ops: 10
 *   Rating: 2
 */
int everyThirdBitOne(void) {
    return 2;
}
/*
 * byteEndSwap - Exchange the first and last byte of a 32bit integer
 *  Examples: byteEndSwap(0x12345678) = 0x78345612
 *            byteEndSwap(0xDEADBEEF) = 0xEFADBEDE
 *  Legal ops: ! ~ & ^ | + << >>
 *  Max ops: 15
 *  Rating: 2
 */
int byteEndSwap(int x) {
    return 2;
}
/*
 * numOddBytes(int x) - returns number of bytes whose least significant bit is 1
 *  Examples: numOddBytes(1) = 1
 *            numOddBytes(128) = 0
 *            numOddBytes(-127) = 4
 *  Legal ops: ~ & ^ | + << >>
 *  Max ops: 30
 *  Rating: 2
 */
int numOddBytes(int x) {
    return 2;
}
/* 
 * subOK - Determine if can compute x-y without overflow
 *   Example: subOK(0x80000000,0x80000000) = 1,
 *            subOK(0x80000000,0x70000000) = 0, 
 *   Legal ops: ! ~ & ^ | + << >>
 *   Max ops: 20
 *   Rating: 3
 */
int subOK(int x, int y) {
  return 2;
}
/*
 * squaredOK - Return 1 if x*x can fit in a 32-bit integer, and 0 otherwise.
 *  Examples: squaredOK(10) = 1
 *            squaredOK(1000000) = 0
 *  Legal ops: ! ~ & ^ | + - << >>
 *  Max ops: 20
 *  Rating: 3
 */
int squaredOK(int x) {
    return 2;
}
/*
 * byteParity - compute high bit of bytes for even parity.
 *
 *  Consider an error detection scheme in which the the most
 *  significant bit of each byte is chosen to be 1 if an odd number
 *  of the other 7 bits in the byte are 1, and 0 otherwise. In other
 *  words, when the parity bit is included, each byte will have an
 *  even number of bits set to 1. Given four bytes in an int,
 *  transform x so that the most significant bits are set as parity
 *  bits for the other 7 bits, which should be unchanged.
 *
 *  Examples: byteParity(0x01020304) = 0x81820384
 *            byteParity(0xAC) = 0xAC
 *            byteParity(0xFFFFFFFF) = 0xFFFFFFFF
 *            byteParity(0x67676767) = 0xE7E7E7E7
 *  Legal ops: ! ~ & ^ | + << >>
 *  Max ops: 50
 *  Rating: 4
 */
int byteParity(int x) {
    return 2;
}
/*
 * floatTimesTwo - Return bit-level equivalent of expression f * 2 for
 *   floating point argument f represented as bits uf.
 *
 *   Both the argument and result are passed as unsigned int's, but
 *   they are to be interpreted as the bit-level representations of
 *   single-precision floating point values.
 *   When argument is NaN, return argument.
 *
 *   Legal ops: Any integer/unsigned operations incl. ||, &&. also if, while
 *   Max ops: 50
 *   Rating: 2
 *
 */
unsigned floatTimesTwo(unsigned uf) {
    return 2;
}
/*
 * roundFloat - Return bit-level equivalent of expression round(f) for
 *   floating point argument f.
 *
 *   Both the argument and result are passed as unsigned int's, but
 *   they are to be interpreted as the bit-level representations of
 *   single-precision floating point values.
 *   When argument is NaN, +/- infinity, or +/- 0, return argument.
 *
 *   roundFloat should round values to the nearest integer, with ties
 *   broken by rounding away from zero. In other words if the
 *   fractional part of a number is less than 0.5, the fraction is
 *   removed, i.e. the value is rounded towards zero. If the fraction
 *   part is 0.5 or more, the value is rounded up to the next number
 *   further away from 0.
 *
 *   Example: roundFloat(3.5)  -> 4
 *   Example: roundFloat(3.3)  -> 3
 *   Example: roundFloat(2.5)  -> 3
 *   Example: roundFloat(2.25) -> 2
 *   Example: roundFloat(-2.25)-> -2
 *   Example: roundFloat(-2.5) -> -3
 *   Example: roundFloat(-3.3) -> -3
 *   Example: roundFloat(-3.5) -> -4
 *
 *   Example: roundFloat(75.67)   -> 76
 *   Example: roundFloat(-128.59) -> -129
 *   Example: roundFloat(42.389)  -> 42
 *
 *   Legal ops: Any integer/unsigned operations incl. ||, &&. also if, while
 *   Max ops: 50
 *   Rating: 4
 *
 */
unsigned roundFloat(unsigned uf) {
    return 2;
}
